/*
	$Id: ArrayTypePlateTransfer.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Jari Häkkinen, Gregory Vincic
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Häkkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Plate;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.Map;

/**
<dl>
<dt><b>Base 1 tables</b></dt>
<dd>ArrayTypePlate</dd>
<dt><b>Notes</b></dt>
<dd>
<ul>
<li>Plates associated with non existing ArrayType's will not be migrated,
these are considered bad entries.</li>
</ul>
</dd>
</dl>

@author Gregory, Martin
@version 2.0
@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/

public class ArrayTypePlateTransfer extends Transfer
{
	/**
	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.ArrayTypePlateTransfer");
	
	private ArrayTypeTransfer arrayTypeT = null;
	private MappedPlateTransfer plateT = null;

	/**
	Constructs a new ArrayTypePlateTransfer. Requires a previously finished
	ReporterTransfer.
	
	@param arrayTypeT A finished {@link ReporterTransfer}
	 */
	ArrayTypePlateTransfer(ArrayTypeTransfer arrayTypeT, MappedPlateTransfer plateT)
	{
		log.debug("Checking that necessary transfers have been done before proceeding with ArrayTypePlate");
		if (arrayTypeT != null && arrayTypeT.isDone())
		{
			this.arrayTypeT = arrayTypeT;
			this.plateT = plateT;
		}
		else
		{
			log.debug("Dependency: ArrayTypes haven't been transfered");
			dependencyFailure();
		}
	}

	public void start()
	{
		log.info("Transferring ArrayTypePlates...");
		startProgress(arrayTypeT.getCount(), "ArrayTypePlates");
		DbControl dc = newDbControl();
		for (Map.Entry<Integer, Integer> entry : arrayTypeT.getIdMap().entrySet())
		{
			Integer base2Id = entry.getValue();
			Integer base1Id = entry.getKey();
			ArrayDesign arrayDesign = ArrayDesign.getById(dc, base2Id);
			arrayDesign.setPlateList(getPlateList(dc, base1Id));
			progress.increase();
		}
		commit(dc);
		done();
		log.info("Transferring ArrayTypePlates...DONE");
	}

	/**
	Returns an ArrayList<Plate> for a given Base 1 arrayDesignId.
	
	@param dc {@link DbControl} used to create a new ArrayDesignBlock
	@param arrayDesignId Base 1 ArrayDesign Id
	 */
	private ArrayList<Plate> getPlateList(DbControl dc, int arrayDesignId)
	{
		ArrayList<Plate> plates = new ArrayList<Plate>();
		try
		{
			PreparedStatement ps = prepareStatementFromFile("selectArrayTypePlates");
			ps.setInt(1, arrayDesignId);
			ResultSet rs = ps.executeQuery();
			if (log.isDebugEnabled()) log.debug("Getting plates for ArrayDesign with BASE 2 id: " + arrayDesignId);
			while (rs.next())
			{
				Integer plateId = plateT.getBase2Id(rs.getInt(2));
				if (plateId != null) plates.add(Plate.getById(dc, plateId));
				if (log.isDebugEnabled())
				{
					log.debug(plates.get(plates.size()-1) + " is added to array design");
				}
			}
		}
		catch (Exception e)
		{
			log.warn("Couldn't get plate list");
			if (log.isDebugEnabled())log.debug("\t" + e);
		}
		return plates;
	}
}
