/**
	$Id: PluginSessionControl.java 5319 2010-04-20 08:02:16Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

/**
 	This is a special session control implementation that is used for 
 	executing plugins that prevents them from logging in or out. If
 	the plugin has enabled permissions those permissions are also loaded and
 	taken into account when checking access to items.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2010-04-20 10:02:16 +0200 (Tue, 20 Apr 2010) $
*/
class PluginSessionControl
	extends SessionControl
{

	private final SessionControl parent;

	/**
		Create a new plugin session control
		@param id The ID of the session control
		@param parent The parent session control which settings and permissions
			should be copied from
		@param plugin The plugin that is executing
	*/
	PluginSessionControl(String id, SessionControl parent, PluginDefinition plugin, Job job)
	{
		super(id, parent, plugin, job);
		this.parent = parent;
	}

	/**
		@throws PermissionDeniedException Always
	*/
	@Override
	void login(Keyring keyring)
	{
		throw new PermissionDeniedException("login");
	}

	/**
		@throws PermissionDeniedException Always
	*/
	@Override
	public synchronized void login(String login, String password, String comment, boolean encryptedPassword) 
		throws ItemNotFoundException, PermissionDeniedException, InvalidPasswordException, BaseException
	{
		throw new PermissionDeniedException("login");
	}

	/**
		Do nothing, a plugin is not allowed to log out.
	*/
	@Override
	public synchronized void logout() 
		throws BaseException
	{}
	
	/**
		@throws PermissionDeniedException Always
	*/
	@Override
	public SessionControl impersonateLogin(int userId, String comment) 
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		throw new PermissionDeniedException("impersonateLogin");
	}

	/**
		@throws PermissionDeniedException Always
	*/
	@Override
	public SessionControl impersonateLogin(Ownable item, String comment)
	{
		throw new PermissionDeniedException("impersonateLogin");
	}


	/**
		Create a new session control for executing a plugin using the same
		parent as this session control has.
	*/
	public PluginSessionControl getPluginSessionControl(PluginDefinition plugin, Job job)
	{
		return Application.newPluginSessionControl(parent, plugin, job);
	}

}
