/*
	$Id: BioAssaySetData.java 4910 2009-04-29 09:05:20Z nicklas $
	
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Set;
import java.util.Map;
import java.util.HashMap;

/**
	This class holds information about a bioassayset.
	
	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.BioAssaySet
	@see <a href="../../../../../../html/developerdoc/api/data_api.experiments.html">Experiments and analysis overview</a>
	@base.modified $Date: 2009-04-29 11:05:20 +0200 (Wed, 29 Apr 2009) $
	@hibernate.class table="`BioAssaySets`" lazy="true"
*/
public class BioAssaySetData
	extends BasicData
	implements NameableData, RemovableData, AnnotatableData, FileStoreEnabledData
{
	public BioAssaySetData()
	{}

	/*
		From the NameableData interface
		-------------------------------------------
	*/
	private String name;
	public String getName()
	{
		return name;
	}

	public void setName(String name)
	{
		this.name = name;
	}

	private String description;
	public String getDescription()
	{
		return description;
	}

	public void setDescription(String description)
	{
		this.description = description;
	}
	// -------------------------------------------
	/*
		From the RemovableData interface
		-------------------------------------------
	*/
	private boolean removed;
	public boolean isRemoved()
	{
		return removed;
	}
	public void setRemoved(boolean removed)
	{
		this.removed = removed;
	}
	// -------------------------------------------
	/*
		From the AnnotatableData interface
		-------------------------------------------
	*/
	private AnnotationSetData annotationSet;
	/**
		@since 2.2
	*/
	public AnnotationSetData getAnnotationSet()
	{
		return annotationSet;
	}
	/**
		@since 2.2
	*/
	public void setAnnotationSet(AnnotationSetData annotationSet)
	{
		this.annotationSet = annotationSet;
	}
	// -------------------------------------------
	/*
		From the FileStoreEnabledData interface
		-------------------------------------------
	*/
	private FileSetData fileSet;
	public FileSetData getFileSet()
	{
	   return fileSet;
	}
	public void setFileSet(FileSetData fileSet)
	{
	   this.fileSet = fileSet;
	}
	// -------------------------------------------

	private ExperimentData experiment;
	/**
		Get the experiment this bioassayset belongs to.
		@hibernate.many-to-one column="`experiment_id`" not-null="true" outer-join="false" update="false"
	*/
	public ExperimentData getExperiment()
	{
		return experiment;
	}
	public void setExperiment(ExperimentData experiment)
	{
		this.experiment = experiment;
	}

	private TransformationData transformation;
	/**
		Get the transformation that created this bioassayset.
		@hibernate.many-to-one column="`transformation_id`" not-null="true" outer-join="false"  update="false"
	*/
	public TransformationData getTransformation()
	{
		return transformation;
	}
	public void setTransformation(TransformationData transformation)
	{
		this.transformation = transformation;
	}

	private DataCubeLayerData dataCubeLayer;
	/**
		Get the layer this bioassayset stores it's data in.
		@hibernate.many-to-one column="`datacubelayer_id`" not-null="true" outer-join="false" 
			update="false"
	*/
	public DataCubeLayerData getDataCubeLayer()
	{
		return dataCubeLayer;
	}
	public void setDataCubeLayer(DataCubeLayerData dataCubeLayer)
	{
		this.dataCubeLayer = dataCubeLayer;
	}
	
	private DataCubeFilterData dataCubeFilter;
	/**
		Get the filter used by this bioassayset. Null = no filter
		@hibernate.many-to-one column="`datacubefilter_id`" not-null="false" outer-join="false" 
			update="false" cascade="save-update"
	*/
	public DataCubeFilterData getDataCubeFilter()
	{
		return dataCubeFilter;
	}
	public void setDataCubeFilter(DataCubeFilterData dataCubeFilter)
	{
		this.dataCubeFilter = dataCubeFilter;
	}

	private int numReporters;
	/**
		The number of unique reporters in this bioassayset.
		@hibernate.property column="`reporters`" type="int" not-null="true" update="false"
	*/
	public int getNumReporters()
	{
		return numReporters;
	}
	public void setNumReporters(int numReporters)
	{
		this.numReporters = numReporters;
	}

	private int numSpots;
	/**
		The number of unique spots in this bioassayset.
		@hibernate.property column="`spots`" type="int" not-null="true" update="false"
	*/
	public int getNumSpots()
	{
		return numSpots;
	}
	public void setNumSpots(int numSpots)
	{
		this.numSpots = numSpots;
	}

	private int numFileReporters;
	/**
		The number of unique reporters as stored in files in this bioassayset.
		@hibernate.property column="`file_reporters`" type="int" not-null="true"
		@since 2.8
	*/
	public int getNumFileReporters()
	{
		return numFileReporters;
	}
	/**
		@since 2.8
	*/
	public void setNumFileReporters(int numFileReporters)
	{
		this.numFileReporters = numFileReporters;
	}

	private int numFileSpots;
	/**
		The number of unique spots in this bioassayset.
		@hibernate.property column="`file_spots`" type="int" not-null="true"
		@since 2.8
	*/
	public int getNumFileSpots()
	{
		return numFileSpots;
	}
	/**
		@since 2.8
	*/
	public void setNumFileSpots(int numFileSpots)
	{
		this.numFileSpots = numFileSpots;
	}
	
	private Set<BioAssayData> bioAssays;
	/**
		This is the inverse end.
		@see BioAssayData#getBioAssaySet()
		@hibernate.set lazy="true" inverse="true" cascade="delete"
		@hibernate.collection-key column="`bioassayset_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.BioAssayData"
	*/
	Set<BioAssayData> getBioAssays()
	{
		return bioAssays;
	}
	void setBioAssays(Set<BioAssayData> bioAssays)
	{
		this.bioAssays = bioAssays;
	}

	private Set<TransformationData> transformations;
	/**
		This is the inverse end.
		@see TransformationData#getSource()
		@hibernate.set lazy="true" inverse="true" cascade="delete"
		@hibernate.collection-key column="`bioassayset_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.TransformationData"
	*/
	Set<TransformationData> getTransformations()
	{
		return transformations;
	}
	void setTransformations(Set<TransformationData> transformations)
	{
		this.transformations = transformations;
	}

	private Map<ExtraValueTypeData, ExtraValueData> extraValues;
	/**
		The extra values calculated for this bioassayset. This
		is the inverse end. This method is public since we need it for
		reading extra values from the bioassayset. The map should never
		be modified by the core.

		@see ExtraValueData#getBioAssaySet()
		@see ExtraValueData#getExtraValueType()
		@hibernate.map lazy="true" cascade="all" inverse="true"
		@hibernate.collection-key column="`bioassayset_id`"
		@hibernate.index-many-to-many column="`extravaluetype_id`" class="net.sf.basedb.core.data.ExtraValueTypeData"
		@hibernate.collection-one-to-many column="`extravalue_id`" 
			class="net.sf.basedb.core.data.ExtraValueData"
	*/
	public Map<ExtraValueTypeData, ExtraValueData> getExtraValues()
	{
		if (extraValues == null) extraValues = new HashMap<ExtraValueTypeData, ExtraValueData>();
		return extraValues;
	}
	void setExtraValues(Map<ExtraValueTypeData, ExtraValueData> extraValues)
	{
		this.extraValues = extraValues;
	}
	
	private int intensityTransform;
	/**
		A flag indicating how the spot intensities has been transformed
		before it was stored in the database:
		0 = no transformation; 1 = 2-log; 2 = 10-log 
		@hibernate.property column="`intensity_transform`" type="int" not-null="true"
		@since 2.12
	*/
	public int getIntensityTransform()
	{
		return intensityTransform;
	}
	/**
		@since 2.12
	*/
	public void setIntensityTransform(int intensityTransform)
	{
		this.intensityTransform = intensityTransform;
	}
	
}
