/**
	$Id: BioPlateEventData.java 5523 2010-12-02 13:35:04Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

/**
 	This class holds information about a bioplate event. An
 	event can involve one or more bioplate participants. It is
 	also possible to link {@link BioMaterialEventData} items to 
 	a bioplate event (via a participant). Some information (eg. 
 	protocol, hardware, etc. is then linked so that an update on
 	the bioplate event is automatically transfered to all biomaterial
 	events. 
  	
  	@author Nicklas
 	@since 2.17
 	@see net.sf.basedb.core.BioPlateEvent
	@see <a href="../../../../../../html/developerdoc/api/data_api.biomaterials.html">Biomaterials overview</a>
	@base.modified $Date: 2010-12-02 14:35:04 +0100 (Thu, 02 Dec 2010) $
	@hibernate.class table="`BioPlateEvents`" lazy="true"
 */
public class BioPlateEventData 
	extends CommonData
	implements LoggableData, RegisteredData
{
	public BioPlateEventData()
	{}

	/*
		From the RegisteredData interface
		---------------------------------
	*/
	private Date entryDate;
	@Override
	public Date getEntryDate()
	{
		return entryDate;
	}
	public void setEntryDate(Date entryDate)
	{
		this.entryDate = entryDate;
	}
	// -----------------------------------
	
	private BioPlateEventTypeData eventType;
	/**
		The type of the event. 
		@hibernate.many-to-one column="`bioplateeventtype_id`" not-null="true" outer-join="false"
	*/
	public BioPlateEventTypeData getEventType()
	{
		return eventType;
	}
	public void setEventType(BioPlateEventTypeData eventType)
	{
		this.eventType = eventType;
	}
	
	private Date eventDate;
	/**
		Get the date this event happened in the lab.
		@hibernate.property column="`event_date`" type="date" not-null="false"
	*/
	public Date getEventDate()
	{
		return eventDate;
	}
	public void setEventDate(Date eventDate)
	{
		this.eventDate = eventDate;
	}
	
	private ProtocolData protocol;
	/**
		Get the protocol used in this event.
		@hibernate.many-to-one column="`protocol_id`" not-null="false" outer-join="false"
	*/
	public ProtocolData getProtocol()
	{
		return protocol;
	}
	public void setProtocol(ProtocolData protocol)
	{
		this.protocol = protocol;
	}

	private HardwareData hardware;
	/**
	 	Get the hardware used in this event
	 	@hibernate.many-to-one column="`hardware_id`" not-null="false" outer-join="false"
	*/
	public HardwareData getHardware()
	{
		return this.hardware;
	}
	public void setHardware(HardwareData hardware)
	{
		this.hardware = hardware;
	}
	
	private Set<BioPlateEventParticipantData> participants;
	/**
		This is the inverse end.
		@see BioPlateEventParticipantData#getEvent()
		@hibernate.set lazy="true" inverse="true" cascade="delete"
		@hibernate.collection-key column="`event_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.BioPlateEventParticipantData"
	*/
	public Set<BioPlateEventParticipantData> getParticipants()
	{
		if (participants == null)
		{
			participants = new HashSet<BioPlateEventParticipantData>();
		}
		return participants;
	}
	void setParticipants(Set<BioPlateEventParticipantData> participants)
	{
		this.participants = participants;
	}


}