/*
  $Id: FileAttachableData.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

/**
	A fileattachable item is an item which can have a <code>File</code> 
	attached to it.
	<p>
	<p>
	This interface defines Hibernate database mapping for the
	<code>file</code> property to the database column <code>file_id</code>.
	If a subclass wants to map the property to another column, 
	it should override the {@link #getFile()} method and add a 
	Hibernate tag in the comment.
	<p>
	
	<b>Reference implementation</b><br>
	<pre class="code">
private FileData file;
public FileData getFile()
{
   return file;
}
public void setFile(FileData file)
{
   this.file = file;
}
</pre>

	@author enell
	@version 2.0
	@see BasicData
	@see <a href="../../../../../../development/overview/data/files.html">File and directory overview</a>
	@see <a href="../../../../../../html/developerdoc/api/data_api.basic.html">Basic classes and interfaces</a>
*/
public interface FileAttachableData
	extends IdentifiableData
{
	/**
		Get the file that is attached to the item.
		@return A {@link FileData} object or null if no file is attached
		@hibernate.many-to-one column="`file_id`" not-null="false" outer-join="false"
	*/
	public FileData getFile();

	/**
		Attach a file to the item. Null is allowed.
	*/
	public void setFile(FileData file);
}
