/*
 $Id: HybridizationData.java 5119 2009-10-08 08:46:51Z nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.basedb.core.data;

import java.util.Set;

/**
	This class holds information about an hybridization.
	
	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.Hybridization
	@see <a href="../../../../../../html/developerdoc/api/data_api.rawdata.html">Hybridzations and raw data overview</a>
	@base.modified $Date: 2009-10-08 10:46:51 +0200 (Thu, 08 Oct 2009) $
	@hibernate.class table="`Hybridizations`" lazy="true"
*/
public class HybridizationData 
	extends AnnotatedData
{

	public HybridizationData()
	{}

	private ArraySlideData arraySlide;

	/**
		Get the array slide.
		@hibernate.many-to-one column="`arrayslide_id`" not-null="false" unique="true" outer-join="false"
	*/
	public ArraySlideData getArraySlide()
	{
		return arraySlide;
	}

	public void setArraySlide(ArraySlideData arraySlide)
	{
		this.arraySlide = arraySlide;
	}
	
	private BioMaterialEventData creationEvent;
	/**
		Get the event that created this hybidization. Mapping is defined
		in hibernate-properties-HybridizationData.xml since XDoclet cannot
		generate it.
	*/
	public BioMaterialEventData getCreationEvent()
	{
		return creationEvent;
	}
	void setCreationEvent(BioMaterialEventData creationEvent)
	{
		this.creationEvent = creationEvent;
	}
	
	private int numArrays;
	/**
		The number of arrays that was hybridized. Shold be <= ArrayDesign.numArrays
		@hibernate.property column="`num_arrays`" type="int" not-null="true"
		@since 2.6
	*/
	public int getNumArrays()
	{
		return numArrays;
	}
	public void setNumArrays(int numArrays)
	{
		this.numArrays = numArrays;
	}

	private Set<ScanData> scans;
	/**
		This is the inverse end.
		@see ScanData#getHybridization()
		@hibernate.set lazy="true" inverse="true"
		@hibernate.collection-key column="`hybridization_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.ScanData"
	*/
	Set<ScanData> getScans()
	{
		return scans;
	}
	void setScans(Set<ScanData> scans)
	{
		this.scans = scans;
	}

}
