/*
  $Id: SettingData.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Johan Enell, Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

/**
	@author enell
	@version 2.0
	@see net.sf.basedb.core.Setting
	@see <a href="../../../../../../html/developerdoc/api/data_api.clients.html">Session and client overview</a>
*/
public abstract class SettingData
	extends BasicData
{
	public SettingData()
	{}

	/**
		The maximum length of the name of the setting that can be
		stored in the database.
		@see #setName(String)
	*/
	public static final int MAX_NAME_LENGTH = 255;
	private String name;
	/** 
		Used by Hibernate to link with setting name.
		@hibernate.property type="string"
		@hibernate.column name="`name`" length="255" not-null="true" unique-key="uniquesetting"
	*/
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}

	/**
		The maximum length of the value of the setting that can be
		stored in the database.
		@see #setValue(String)
	*/
	public static final int MAX_VALUE_LENGTH = 65535;
	private String value;
	/**
		Get the value of this setting.
		@hibernate.property column="`value`" type="text" not-null="true"
	*/
	public String getValue()
	{
		return value;
	}
	public void setValue(String value)
	{
		this.value = value;
	}
}
