/**
	$Id: ImageLogger.java 5384 2010-08-13 08:50:27Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.log.db;


import net.sf.basedb.core.Item;
import net.sf.basedb.core.data.ChangeHistoryDetailData;
import net.sf.basedb.core.data.ImageData;
import net.sf.basedb.core.log.EntityDetails;
import net.sf.basedb.core.log.ChangeType;
import net.sf.basedb.core.log.EntityLogger;
import net.sf.basedb.core.log.LogManager;

/**
	Logger for images that logs changes as
	updates to the scan they belong to.

	@author Nicklas
	@version 2.13
	@base.modified $Date: 2010-08-13 10:50:27 +0200 (Fri, 13 Aug 2010) $
*/
public class ImageLogger
	implements EntityLogger
{

	private final boolean detailedProperties;

	/**
		Creates a new image logger.
		@param detailedProperties If TRUE, the log includes a list with
			the properties that was modified
	*/
	public ImageLogger(boolean detailedProperties)
	{
		this.detailedProperties = detailedProperties;
	}
	
	/*
		From the EntityLogger interface
		-------------------------------
	*/
	@Override
	public void logChanges(LogManager logManager, EntityDetails details)
	{
		ImageData image = (ImageData)details.getEntity();
		ChangeHistoryDetailData change = new ChangeHistoryDetailData();
		change.setChangeType(ChangeType.UPDATE.getValue());
		change.setItemId(image.getScan().getId());
		change.setItemType(Item.SCAN.getValue());
		change.setChangeInfo(details.getChangeType().was("Image") + ": " + image.getName());
		if (details.getChangeType() == ChangeType.UPDATE && detailedProperties)
		{
			String moreInfo = details.getModifiedProperties(null, ", ", null);
			if (moreInfo != null) 
			{
				change.setChangeInfo(change.getChangeInfo() + " (" + moreInfo + ")");
			}
		}
		((DbLogManager)logManager).logChangeDetails(change);
	}
	// -------------------------------
}
