package net.sf.basedb.core.plugin;

import net.sf.basedb.core.PluginParameter;

/**
	A helper class that wraps a plug-in parameter object with a namespace. 
	The parent parameter is cloned to a parameter with the namespace
	pre-pended to the name. Eg, the 'name' value is replaced with the namespace 
	+ dot + the original name. For example, if namespace="export" and
	name="toDir", the resulting name is "export.toDir". 
	<p>
	The main usage for this class is when a master plug-in needs to use
	other child plug-ins. By wrapping the child parameters with an instance 
	of this class it makes it possible to mix parameters from several child
	plug-ins without having to worry about name clashes.
	
	@author nicklas
	@since 2.15
	@base.modified $Date $
	@see NamespaceParameterValuesWrapper
	@see NamespaceRequestWrapper
*/
public class NamespacePluginParameter<T>
	extends PluginParameter<T>
{


	/**
		Create a new parameter wrapper for the plug-in parameter.
		@param parent The plug-in parameter to wrap, null is not allowed
		@param namespace The namespace, null is not allowed
	*/
	public NamespacePluginParameter(PluginParameter<T> parent, String namespace)
	{
		super(namespace + "." + parent.getName(), 
				parent.getLabel(), parent.getDescription(), 
				parent.getDefaultValue(), parent.getParameterType());
		if (namespace == null) throw new NullPointerException("namespace");
	}
	
}
