/*
	$Id: BetweenRestriction.java 5320 2010-04-21 09:48:29Z nicklas $

	Copyright (C) 2005 Samuel Andersson, Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Arrays;
import java.util.Collection;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.util.EqualsHelper;

/**
	Compare if one expression falls between to other expressions: e BETWEEN low AND high.

	@author Samuel, Nicklas
	@version 2.0
	@see Restrictions#between(Expression, Expression, Expression)
	@base.modified $Date: 2010-04-21 11:48:29 +0200 (Wed, 21 Apr 2010) $
*/
public class BetweenRestriction
	implements Restriction
{
	private final Expression e;
	private final Expression low;
	private final Expression high;

	BetweenRestriction(Expression e, Expression low, Expression high)
	{
		assert e != null : "e == null";
		assert low != null : "low == null";
		assert high != null : "high == null";
		this.e = e;
		this.low = low;
		this.high = high;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		return "(" + low.toQl(query, dc) + " <= " + e.toQl(query, dc) + " AND " + e.toQl(query, dc) + " < " + high.toQl(query, dc) + ")";
	}
	/**
		@return The expressions
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return Arrays.asList(e, low, high);
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		return "(" + low + "<=" + e + " AND " + e + " < " + high + ")";
	}
	/**
		This restriction is equal to another BetweenRestriction if their 
		child expressions are equal.
	*/
	@Override
	public boolean equals(Object other)
	{
		if (this == other) return true;
		if (other == null || this.getClass() != other.getClass()) return false;
		BetweenRestriction o = (BetweenRestriction)other;
		return EqualsHelper.equals(e, o.e) && EqualsHelper.equals(low, o.low) &&
			EqualsHelper.equals(high, o.high);
	}
	@Override
	public int hashCode()
	{
		return EqualsHelper.hashCode(e, low, high);
	}
	// -------------------------------------------

}

