/*
	$Id: SelectedExpression.java 5320 2010-04-21 09:48:29Z nicklas $

	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Collection;
import java.util.Collections;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.HibernateUtil;
import net.sf.basedb.core.dbengine.DbEngine;
import net.sf.basedb.util.EqualsHelper;

/**
	An expression representing an already selected expression.
	If the select object has an alias, the alias is used otherwise the
	expression is regenerated.

	@author Nicklas
	@version 2.0
	@see Expressions#selected(Select)
	@base.modified $Date: 2010-04-21 11:48:29 +0200 (Wed, 21 Apr 2010) $
*/
public class SelectedExpression
	implements Expression
{

	private final Select select;

	SelectedExpression(Select select)
	{
		this.select = select;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		QuerySection section = query.getQuerySection();
		DbEngine engine = HibernateUtil.getDbEngine();
		String alias = select.getAlias();
		return alias != null && section.supportsColumnAlias(engine) ? 
				alias : select.getExpression().toQl(query, dc);
	}
	/**
		@return The selected expression
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return Collections.singleton(select);
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		String alias = select.getAlias();
		return (alias != null ? alias : select.getExpression().toString());
	}
	/**
		This expression is equal to another SelectedExpression if their 
		child expressions are equal.
	*/
	@Override
	public boolean equals(Object other)
	{
		if (this == other) return true;
		if (other == null || this.getClass() != other.getClass()) return false;
		SelectedExpression o = (SelectedExpression)other;
		return EqualsHelper.equals(select, o.select);
	}
	@Override
	public int hashCode()
	{
		return EqualsHelper.hashCode(select) + getClass().hashCode();
	}
	// -------------------------------------------
}
