/*
	$Id: ConsoleProgressReporter.java 5329 2010-04-29 11:33:19Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
 
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

 	You should have received a copy of the GNU General Public License
 	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util;

import net.sf.basedb.core.ProgressReporter;

/**
	An implementation of the ProgressReporter interface that writes
	all messages to the standard console.
	
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2010-04-29 13:33:19 +0200 (Thu, 29 Apr 2010) $

*/
public class ConsoleProgressReporter 
	implements ProgressReporter
{

	private final boolean useNewLine;
	private boolean needNewLine = false;

	public ConsoleProgressReporter()
	{
		this(true);
	}
	public ConsoleProgressReporter(boolean useNewline)
	{
		this.useNewLine = useNewline;
	}
	
	/*
		From the ProgressReporter interface
		-------------------------------------------
	*/
	public void display(int percent, String message)
	{
		if (needNewLine) System.out.print(useNewLine ? "\n" : "\r");
		System.out.print("["+(percent == -1 ? "unknown" : percent + "%") + "]\t" + 
				(message == null ? "" : message));
		needNewLine = true;
	}
	public void append(String message)
	{
		System.out.print(message);
	}
	// -------------------------------------------

}
