/**
	$Id: MatrixModel.java 5224 2010-01-27 13:03:33Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.bfs;

import java.util.ArrayList;
import java.util.List;

/**
	A model container for BFS data files. This class is primarily
	intended to be used when parsing small data files. It is not
	recommended to use this class with large files since it may use too
	much memory.
	<p>
	After parsing data is available as

	@author Nicklas
	@version 2.15
	@base.modified $Date: 2010-01-27 14:03:33 +0100 (Wed, 27 Jan 2010) $
*/
public class MatrixModel
	implements EventHandler
{
	
	private List<String[]> data;
	private int numColumns = -1;

	/**
		Create a new annotation model.
	*/
	public MatrixModel()
	{
		this.data = new ArrayList<String[]>();
	}
	
	/*
		From the EventHandler interface
		--------------------------------
	*/
	@Override
	public <T> void handleEvent(EventType<T> eventType, T eventData, BfsParser parser)
	{
		if (eventType == DataParser.DATA_EVENT)
		{
			addRow((String[])eventData);
		}
	}
	// ------------------------------------

	/**
		Add a row of data to the model. The number of array elements
		in the array should be the same as the number of columns.
	*/
	public void addRow(String... row)
	{
		data.add(row);
		if (numColumns == -1) numColumns = row.length;
	}
	
	/**
		Get the number of columns.
	*/
	public int getColumnCount()
	{
		return numColumns;
	}
	
	/**
		Get the number of rows.
	*/
	public int getRowCount()
	{
		return data.size();
	}
	
	/**
		Get the data element at a given row and column index.
		@param rowIndex The row index, a value between 0 and 
			{@link #getRowCount()}-1
		@param colIndex The column index, a value between
			0 and {@link #getColumnCount()}-1
		@return The data at the given location
	*/
	public String getData(int rowIndex, int colIndex)
	{
		String[] row = getRowByIndex(rowIndex);
		return row[colIndex];
	}
	
	public String[] getRowByIndex(int rowIndex)
	{
		return data.get(rowIndex);
	}


}
