/**
$Id: TarFilePacker.java 4515 2008-09-11 20:08:14Z jari $

Copyright (C) 2007 Nicklas Nordborg, Martin Svensson

This file is part of BASE - BioArray Software Environment.
Available at http://base.thep.lu.se/

BASE is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

BASE is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.util.zip;

import net.sf.basedb.util.FileUtil;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import com.ice.tar.TarEntry;
import com.ice.tar.TarOutputStream;

/**
 	This class packs files into a TAR archive.
  	@author martin, Nicklas
  	@version 2.4
  	@base.modified $Date: 2007-07-11 15:45:01 +0200 (Mon, 02 Jul 2007) $  	
*/
public class TarFilePacker 
	implements FilePacker
{
	
	private TarOutputStream tar;
	
	/**
		Creates a new TarFilePacker. 
	*/
	public TarFilePacker()
	{}
	
	/*
		From the FilePacker interface
		-------------------------------------------
	*/
	/**
		@return "TAR-archive (.tar)"
	*/
	public String getDescription()
	{
		return "TAR-archive (.tar)";
	}
	/**
	  @return "tar"
	*/
	public String getFileExtension()
	{
		return "tar";
	}
	/**
	  @return "application/x-tar"
	*/
	public String getMimeType()
	{
		return "application/x-tar";
	}
	/**
	 * Wrap the output stream in a {@link TarOutputStream}.
	 */
	public void setOutputStream(OutputStream out)
		throws IOException
	{
		this.tar = new TarOutputStream(out);
	}
	/**
		Create a new {@link TarEntry} and write the compressed
		data to it.
	*/
	public void pack(String entryName, InputStream in, long size, long lastModified)
		throws IOException
	{
		boolean isDirectory = in == null;
		if (isDirectory && !entryName.endsWith("/")) entryName += "/";
		TarEntry entry = new TarEntry(entryName);
		if (lastModified > 0) entry.setModTime(lastModified);
		entry.setSize(size);
		TarUtil.putNextEntry(entry, tar);
		if (!isDirectory) FileUtil.copy(in, tar);
		tar.closeEntry();
	}
	/**
		Finish the TAR file.
	*/
	public void close()
		throws IOException
	{
		tar.close();
		tar = null;
	}
	// -------------------------------------------

}
