/*
	$Id: TestGenericOverview.java 4768 2009-02-18 13:26:21Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import java.util.List;

import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.HardwareType;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.ProtocolType;
import net.sf.basedb.core.Quantity;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.core.SoftwareType;
import net.sf.basedb.core.SystemItems;
import net.sf.basedb.core.Type;
import net.sf.basedb.util.overview.Failure;
import net.sf.basedb.util.overview.GenericOverview;
import net.sf.basedb.util.overview.Node;
import net.sf.basedb.util.overview.OverviewUtil;
import net.sf.basedb.util.overview.loader.BasicItemNodeLoaderFactory;

public class TestGenericOverview
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing generic overview");
		// Create support items
		// Annotation types
		int annotationType = TestAnnotationType.test_create("Source", Type.STRING, 0, 
				null, null, Item.RAWBIOASSAY, 1, new String[] {"alfa", "beta"}, true);

		// Protocols
		int scanningProtocol = TestProtocol.test_create(
				SystemItems.getId(ProtocolType.SCANNING), "Scan protocol", false);
		int featureProtocol = TestProtocol.test_create(
				SystemItems.getId(ProtocolType.FEATURE_EXTRACTION), "Feature protocol", false);
		int power = SystemItems.getId(Quantity.POWER);
		int milliWatt = TestUnit.test_load_by_symbol(power, "mW");
		int scanPower = TestAnnotationType.test_create("Power", Type.FLOAT, milliWatt, 
				null, null, Item.SCAN, 1, null, false);
		TestProtocol.test_set_protocol_parameter(scanningProtocol, scanPower);

		// Hardware and software
		int scanner = TestHardware.test_create(
				SystemItems.getId(HardwareType.SCANNER), "Overview scanner", false);
		int software = TestSoftware.test_create(
				SystemItems.getId(SoftwareType.FEATURE_EXTRACTION), "Overview software", false);
		
		// Data files
		int genePixFile = TestFile.test_create("data/test.rawdata.import.txt", false, false);
		int celFile = TestFile.test_create("data/test.affymetrix.cel", false, false);
		int reporterMap = TestFile.test_create("data/test", false, false);

		// Create main items
		
		// Biomaterial
		int biosource = TestBioSource.test_create("Test GO", false);
		int sample1 = TestSample.test_create(biosource, "Test GO #1", false);
		int sample2 = TestSample.test_create(biosource, "Test GO #2", false);
		int extract1 = TestExtract.test_create(sample1, false);
		int extract2 = TestExtract.test_create(sample2, false);
		int extractPooled = TestExtract.test_create_pooled(extract1, extract2);
		
		int cy3 = TestLabel.test_load("cy3");
		int cy5 = TestLabel.test_load("cy5");
		int labeledExtract1 = TestLabeledExtract.test_create(extract1, cy3, false);
		int labeledExtract2 = TestLabeledExtract.test_create(extract2, cy3, false);
		int labeledExtract1Ref = TestLabeledExtract.test_create(extractPooled, cy5, false);
		int labeledExtract2Ref = TestLabeledExtract.test_create(extractPooled, cy5, false);

		// Array design
		int arrayDesign = TestArrayDesign.test_create(Platform.GENERIC, false);
		TestArrayDesign.test_set_file(arrayDesign, DataFileType.GENERIC_REPORTER_MAP, reporterMap);
		
		// Array batch & slide
		int arrayBatch = TestArrayBatch.test_create(arrayDesign, false);
		int arraySlide = TestArraySlide.test_create(arrayBatch, false);
		
		// Hybridization
		int hyb = TestHybridization.test_create(arraySlide, false);
		TestHybridization.test_add_labeledextract(hyb, labeledExtract1, 1, null);
		TestHybridization.test_add_labeledextract(hyb, labeledExtract1Ref, 1, null);
		TestHybridization.test_add_labeledextract(hyb, labeledExtract2, 2, null);
		TestHybridization.test_add_labeledextract(hyb, labeledExtract2Ref, 2, null);

		// Scan
		int scan = TestScan.test_create(hyb, scanner, scanningProtocol, false);
		TestAnnotation.test_annotatate(Item.SCAN, scan, scanPower, 0, 15.0f);
		TestScan.test_add_image(scan);
		
		// Raw bioassay #1
		int rawBioAssay1 = TestRawBioAssay.test_create(Platform.GENERIC, "genepix", "RBA #1", scan, software, featureProtocol, arrayDesign, 1, false);
		TestRawBioAssay.test_set_file(rawBioAssay1, DataFileType.GENERIC_RAW_DATA, genePixFile);
		TestRawBioAssay.test_set_file(rawBioAssay1, DataFileType.AFFYMETRIX_CEL, celFile);
		TestAnnotation.test_annotatate(Item.RAWBIOASSAY, rawBioAssay1, annotationType, 0, "alfa");

		// Raw bioassay #2
		int rawBioAssay2 = TestRawBioAssay.test_create(Platform.GENERIC, "genepix", "RBA #2", scan, software, scanningProtocol, arrayDesign, 2, false);
		TestRawBioAssay.test_set_file(rawBioAssay2, DataFileType.GENERIC_RAW_DATA, genePixFile);

		// Experiment
		int experiment = TestExperiment.test_create("genepix", false);
		TestExperiment.test_add_rawbioassay(experiment, rawBioAssay1, rawBioAssay2);
		TestExperiment.test_add_experimental_factors(experiment, scanPower, annotationType);

		// Show some overviews
		test_overview(Item.BIOSOURCE, biosource);
		test_overview(Item.HYBRIDIZATION, hyb);
		test_overview(Item.ARRAYDESIGN, arrayDesign);
		test_overview(Item.EXPERIMENT, experiment);
		
		// Delete items
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		// Main items
		TestExperiment.test_delete(experiment);
		TestRawBioAssay.test_delete(rawBioAssay1);
		TestRawBioAssay.test_delete(rawBioAssay2);
		TestScan.test_delete(scan);
		TestHybridization.test_delete(hyb);
		TestArraySlide.test_delete(arraySlide);
		TestArrayBatch.test_delete(arrayBatch);
		TestArrayDesign.test_delete(arrayDesign);
		TestLabeledExtract.test_delete(labeledExtract1);
		TestLabeledExtract.test_delete(labeledExtract1Ref);
		TestLabeledExtract.test_delete(labeledExtract2);
		TestLabeledExtract.test_delete(labeledExtract2Ref);
		TestExtract.test_delete(extractPooled);
		TestExtract.test_delete(extract1);
		TestExtract.test_delete(extract2);
		TestSample.test_delete(sample1);
		TestSample.test_delete(sample2);
		TestBioSource.test_delete(biosource);
		
		// Support items
		TestSoftware.test_delete(software);
		TestHardware.test_delete(scanner);
		TestProtocol.test_delete(featureProtocol);
		TestProtocol.test_delete(scanningProtocol);
		TestFile.test_delete(celFile);
		TestFile.test_delete(genePixFile);
		TestFile.test_delete(reporterMap);
		TestAnnotationType.test_delete(scanPower);
		TestAnnotationType.test_delete(annotationType);
		
		write("++Testing generic overview "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	

	
	static void write(String message)
	{
		System.out.println(message);
	}
	static void write(GenericOverview go)
	{
		if (TestUtil.getSilent()) return;
		write("--Generating overview...");
		write(go.getRootNode(), "");
		int i = 0;
		for (Failure f : go.getFailures())
		{
			++i;
			write("--failure " + i + ": " + f.getMessage() + " on node " + f.getNode().getTitle());
		}
	}
	static void write(Node node, String indent)
	{
		write("  " + indent + node.getName() + ": " + node.getTitle());
		List<Node> children = node.getChildren();
		if (children != null)
		{
			for (Node child : children)
			{
				write(child, "".equals(indent) ? "|-" : "| " + indent);
			}
		}
	}
	
	
	static void test_overview(Item itemType, int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		SessionControl sc = TestUtil.getSessionControl();
		try
		{
			dc = TestUtil.getDbControl();
			BasicItem item = itemType.getById(dc, id);
			
			GenericOverview go = OverviewUtil.getNewOverview(dc, item, null);
			((BasicItemNodeLoaderFactory)go.getNodeLoaderFactory()).setUseNullLoaderIfNotFound(true);
			go.expand(dc, go.getRootNode(), true);
			write(go);
			
			write("--Load generic overview [root=" + item + "] OK");
		}
		catch (Throwable ex)
		{
			write("--Load generic overview [root =" + itemType + "(" + id + ") FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	

}
