/*
	$Id: TestReporterMapFlatFileImporter.java 5319 2010-04-20 08:02:16Z nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.ParameterType;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.PluginConfiguration;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.PluginConfigurationRequest;
import net.sf.basedb.core.PluginResponse;
import net.sf.basedb.core.ReporterType;
import net.sf.basedb.core.RequestInformation;
import net.sf.basedb.core.plugin.Response;


public class TestReporterMapFlatFileImporter
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing reporter map import using plugin");
		
		// Import reporters needed for reportermap import
		// Create reporter type and upload file
		int reporterTypeId = TestReporterType.test_create();
		int arrayDesignId = TestArrayDesign.test_create(Platform.GENERIC, false);
		int fileId = TestFile.test_create("data/test.reportermap.import.txt", false, false);
		
		// Create reporter plugin configuration and job
		int reporterPluginDefinitionId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.ReporterFlatFileImporter");
		int reporterPluginConfigurationId = test_create_configuration_reporter(reporterPluginDefinitionId, reporterTypeId);
		int jobId = TestReporterFlatFileImporter.test_create_job(reporterPluginConfigurationId, fileId, "create");
		
		// Execute job
		ok &= TestJob.test_execute(jobId, false);

		TestJob.test_delete(jobId);
		TestPluginConfiguration.test_delete(reporterPluginConfigurationId);
		
		// ReporterMap import
		int reporterMapPluginDefinitionId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.ReporterMapFlatFileImporter");
		int reporterMapPluginConfigurationId = test_create_configuration_reporter_map(reporterMapPluginDefinitionId);
		jobId = test_create_map_job(reporterMapPluginConfigurationId, fileId, arrayDesignId);
		
		// Execute job
		ok &= TestJob.test_execute(jobId, false);
		
		// Print result
		TestArrayDesign.write_feature_header();
		TestArrayDesign.test_list_features(arrayDesignId, -1);

		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		
		TestArrayDesign.test_delete(arrayDesignId);
		TestJob.test_delete(jobId);
		TestPluginConfiguration.test_delete(reporterMapPluginConfigurationId);
		TestReporter.test_delete();
		TestFile.test_delete(fileId);
		TestReporterType.test_delete(reporterTypeId);

		write("++Testing reporter map import using plugin "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create_configuration_reporter(int pluginDefinitionId, int reporterTypeId)
	{
		if (pluginDefinitionId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.PLUGINCONFIGURATION)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginDefinition pd = PluginDefinition.getById(dc, pluginDefinitionId);
			PluginConfiguration pc = pd.newPluginConfiguration();
			pc.setName(pd.getName());
			
			PluginConfigurationRequest request = pc.configure();

			write_request_information(request.getRequestInformation());
			
			request.setParameterValue("dataSplitterRegexp", "\\t");
			request.setParameterValue("dataHeaderRegexp", "\"Block\"\\t\"Column\"\\t\"Row\"\\t\"ID\"\\t\"Name\"\\t\"Symbol\"");
			request.setParameterValue("reporterIdColumnMapping", "\\ID\\");
			request.setParameterValue("nameColumnMapping", "\\Name\\");
			request.setParameterValue("symbolColumnMapping", "\\Symbol\\");
			if (reporterTypeId != 0)
			{
				request.setParameterValue("reporterType", ReporterType.getById(dc, reporterTypeId));
			}
			
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(pc);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = pc.getId();
			write("--Create configuration for reporter import OK");
		}
		catch (Throwable ex)
		{
			write("--Create configuration for reporter import FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static int test_create_configuration_reporter_map(int pluginDefinitionId)
	{
		if (pluginDefinitionId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.PLUGINCONFIGURATION)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginDefinition pd = PluginDefinition.getById(dc, pluginDefinitionId);
			PluginConfiguration pc = pd.newPluginConfiguration();
			pc.setName(pd.getName());
			
			PluginConfigurationRequest request = pc.configure();

			write_request_information(request.getRequestInformation());
			
			request.setParameterValue("dataSplitterRegexp", "\"?\\t\"?");
			request.setParameterValue("dataHeaderRegexp", "\"Block\"\\t\"Column\"\\t\"Row\"\\t\"ID\"\\t\"Name\"\\t\"Symbol\"");
			request.setParameterValue("reporterIdColumnMapping", "\\3\\");
			request.setParameterValue("blockColumnMapping", "\\0\\");
			request.setParameterValue("columnColumnMapping", "\\1\\");
			request.setParameterValue("rowColumnMapping", "\\2\\");
			
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(pc);
				dc.commit();
			}
			else if (response.getStatus() == Response.Status.ERROR)
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			else
			{
				throw new BaseException("Unknown status on response: "+response.getStatus());
			}
			id = pc.getId();
			write("--Create configuration for reporter map import OK");
		}
		catch (Throwable ex)
		{
			write("--Create configuration for reporter map import FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static int test_create_job(int pluginConfigurationId, int fileId)
	{
		if (pluginConfigurationId == 0 || fileId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.JOB)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginConfiguration pc = PluginConfiguration.getById(dc, pluginConfigurationId);
			Job j = pc.newJob(null);
			j.setName(pc.getName());
			
			PluginConfigurationRequest request = j.configure(null);
			write_request_information(request.getRequestInformation());
			request.setParameterValue("file", File.getById(dc, fileId));
			request.setParameterValue("updateExisting", false);
			
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(j);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = j.getId();
			write("--Create job for import OK");
		}
		catch (Throwable ex)
		{
			write("--Create job for import FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static int test_create_map_job(int pluginConfigurationId, int fileId, int arrayDesignId)
	{
		if (pluginConfigurationId == 0 || fileId == 0 || arrayDesignId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.JOB)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginConfiguration pc = PluginConfiguration.getById(dc, pluginConfigurationId);
			Job j = pc.newJob(null);
			j.setName(pc.getName());
			
			PluginConfigurationRequest request = j.configure(null);
			write_request_information(request.getRequestInformation());
			request.setParameterValue("file", File.getById(dc, fileId));
			request.setParameterValue("arrayDesign", ArrayDesign.getById(dc, arrayDesignId));
			
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(j);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = j.getId();
			write("--Create job for map import OK");
		}
		catch (Throwable ex)
		{
			write("--Create job for map import FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	

	static void write(String message)
	{
		System.out.println(message);
	}


	static void write_request_information(RequestInformation ri)
	{
		if (TestUtil.getSilent()) return;

		write("Request information for command: "+ri.getCommand());
		write("\tTitle:\t"+ri.getTitle());
		write("\tDescription:\t"+ri.getDescription());
		for (PluginParameter<?> pp : ri.getParameters())
		{
			ParameterType pt = pp.getParameterType();
			write("\tParameter:\t"+pp.getName() + "\t" + pp.getLabel() + "\t" + pt );
		}
	}
}

