import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.SoftwareType;
import net.sf.basedb.core.SystemItems;


/*
 $Id$

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
 */

public class TestSoftwareType
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing software type");
		write_header();
		// Standard tests: load and list
		test_load(SystemItems.getId(SoftwareType.FEATURE_EXTRACTION));
		test_list();
		
		write("++Testing software type "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			SoftwareType st = SoftwareType.getById(dc, id);
			write_item(0, st);
			write("--Load software type OK");
		}
		catch (Throwable ex)
		{
			write("--Load software type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<SoftwareType> l = SoftwareType.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			write("--List software types OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List software types FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription");
			write("-- \t-- \t--------- \t-----------");
		}
	}
	static void write_item(int i, SoftwareType st)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+st.getId()+"\t"+st.getName()+"\t"+
			st.getDescription());
	}
	static void write(String message)
	{
		System.out.println(message);
	}
}
