/*
	$Id: TestTrashcan.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Nameable;
import net.sf.basedb.core.Removable;
import net.sf.basedb.core.Trashcan;

public class TestTrashcan
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing Trashcan");
		write_header();
		test_list();
		write("++Testing Trashcan "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	
	static void test_list()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			List<Removable> l = Trashcan.getItems(dc, (Set<Item>)null, 0, 0);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			write("--List items in trashcan OK");
		}
		catch (Throwable ex)
		{
			write("--List items in trashcan FAILED");
			ex.printStackTrace();
			ok = false;
		}
	}
	
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tType \tName      \tDescription");
			write("-- \t-- \t---- \t--------- \t-----------");
		}
	}

	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void write_item(int i, Removable item)
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.print(i+":\t"+item.getId()+"\t"+item.getType());
			if (item instanceof Nameable)
			{
				Nameable nameable = (Nameable)item;
				String description = nameable.getDescription();
				if (description != null && description.length() > 20) 
				{
					description = description.substring(0, 20) + "...";
				}
				System.out.println("\t" + nameable.getName() + "\t" + description);
			}
			else
			{
				System.out.println("\t" + item.toString());
			}
		}

	}

}

