/* $Id: newjoust.js 5179 2009-11-04 12:04:47Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	the Joust Outliner. The original code has been heavily modified 
	to fit our purposes and coding style. This code is not compatible 
	with the original code and lacks a lot of the functionality of
	the original version.

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	Joust Outliner Version 2.5.4
	(c) Copyright 1996-2001, MITEM (Europe) Ltd. All rights reserved.
	This code may be freely copied and distributed provided that it is 
	accompanied by this header.  For full details of the Joust license, 
	as well as documentation and help, go to http://www.ivanpeters.com/.
*/

var JoustMenu = new JoustMenuClass();
/**
	Represents the entire outline.
*/
function JoustMenuClass()
{
	this.alwaysSendOnClickToSelected = false;
	this.firstItemIndex = -1;
	this.lastItemIndex = -1;
	this.selectedItemIndex = -1;
	this.menuItems = new Array();

	/*
		Draw the outline. 
		@param id The ID of div tag where the outline will be place
	*/
	this.draw = function(id)
	{
		var html = this.drawMenuItems(this.firstItemIndex, '');
		if (id)
		{
			this.div = document.getElementById(id);
			if (this.div) this.div.innerHTML = html;
		}
		else
		{
			document.write(html);
		}
	}
	
	/*
		Create the HTML for drawing the entire tree starting at the
		given index. The method will draw, in this order, the given item, 
		it's children (recursive) and all following items. 
		@param firstIndex The index of the first item
		@param indentString A string that is prepended to the menuitems to create
			the effect of indentation
	*/
	this.drawMenuItems = function(firstIndex, indentString)
	{
		var html = '';
		var menuItem = this.menuItems[firstIndex];
		while (menuItem)
		{
			html += menuItem.draw(indentString);
			var padIcon = IconStore.getIcon(menuItem.noOutlineIcon == true ? null : menuItem.nextItemIndex == -1 ? 'iconBlank' : 'iconLine');
			var padHtml = padIcon == null ? '' : padIcon.getImgTag();
			if (menuItem.firstChildIndex != -1 || menuItem.lazyInit)
			{
				html += '<div class="children" id="children'+menuItem.index+'" style="display: '+(menuItem.isOpen ? 'block' : 'none')+'">';
				menuItem.childIndentation = indentString+padHtml;
				if (menuItem.firstChildIndex != -1) 
				{
					html += this.drawMenuItems(menuItem.firstChildIndex, menuItem.childIndentation);
				}
				html += '</div>\n';
			}
			menuItem = this.menuItems[menuItem.nextItemIndex];
		}
		return html;
	}
	
	/**
		Adds a menu item to the menu
		@param afterItemIndex The index of the item that should be before the new item,
			or -1 to place the new item at the end of the root level
		@param iconName The name of this item document icon
		@param text The text of this item
		@param onclick A string containing a JavaScript statement that is evaluated when the user clicks on the menu item
		@param tooltip A tooltip text
		@return The index of the new item
	*/
	this.addMenuItem = function(afterItemIndex, iconName, text, onclick, tooltip, externalId, isOpen, lazyInit) 
	{
		var afterItem = this.menuItems[afterItemIndex];
		if (!afterItem) afterItem = this.menuItems[this.lastItemIndex];
		var beforeItem = afterItem ? this.menuItems[afterItem.nextItemIndex] : null;
		
		var nextItemIndex = beforeItem ? beforeItem.index : -1;
		var previousItemIndex = afterItem ? afterItem.index : -1;
		var parentItemIndex = afterItem ? afterItem.parentItemIndex : -1;
		var insertIndex = this.menuItems.length;

		var menuItem = new MenuItem(this, insertIndex, iconName, text, onclick, tooltip, nextItemIndex, previousItemIndex, parentItemIndex, externalId, isOpen, lazyInit);
		this.menuItems[insertIndex] = menuItem;
		if (!this.menuItems[externalId]) this.menuItems[externalId] = menuItem;
		if (afterItem) 
		{
			afterItem.nextItemIndex = insertIndex;
			if (beforeItem) 
			{
				beforeItem.previousItemIndex = insertIndex;
			}
			else
			{
				//afterItem.lastChildIndex = insertIndex;
				if (parentItemIndex == -1) this.lastItemIndex = insertIndex;
			}
		}
		else
		{
			this.firstItemIndex = insertIndex;
			this.lastItemIndex = insertIndex;
		}
		return insertIndex;
	}

	this.addLazyChildItem = function(parentItemIndex, iconName, text, onclick, tooltip, externalId, lazyInit)
	{
		return this.addChildItem(parentItemIndex, iconName, text, onclick, tooltip, externalId, false, lazyInit);
	}
	
	/**
		Adds a child item to another menu item. The new item is inserted last
		among the children. Use addMenuItem to insert a new menu item in another
		position.
		@param parentItemIndex The index of the parent item
		@param iconName The name of this item document icon
		@param text The text of this item
		@param onclick A string containing a JavaScript statement that is evaluated when the user clicks on the menu item
		@param tooltip A tooltip text
		@return The index of the new item
	*/
	this.addChildItem = function(parentItemIndex, iconName, text, onclick, tooltip, externalId, isOpen, lazyInit)
	{
		var parentItem = this.menuItems[parentItemIndex];
		if (!parentItem)
		{
			return this.addMenuItem(-1, iconName, text, onclick, tooltip, externalId, isOpen, lazyInit);
		}
		var afterItem = this.menuItems[parentItem.lastChildIndex];
		if (afterItem)
		{
			parentItem.lastChildIndex = this.addMenuItem(parentItem.lastChildIndex, iconName, text, onclick, tooltip, externalId, isOpen, lazyInit);
			return parentItem.lastChildIndex;
		}
		var insertIndex = this.menuItems.length;
		var menuItem = new MenuItem(this, insertIndex, iconName, text, onclick, tooltip, -1, -1, parentItemIndex, externalId, isOpen, lazyInit);
		this.menuItems[insertIndex] = menuItem;
		if (!this.menuItems[externalId]) this.menuItems[externalId] = menuItem;
		parentItem.firstChildIndex = insertIndex;
		parentItem.lastChildIndex = insertIndex;
		return insertIndex;
	}

	/**
		Toggle the open/closed status of a menu item
		@param menuItemIndex The index of the menu item to toggle
	*/
	this.toggle = function(menuItemIndex)
	{
		var menuItem = this.menuItems[menuItemIndex];
		if (!menuItem) return;
		// Switch the open/closed status and hide or show the children
		if (menuItem.lazyInit)
		{
			var lazyInit = menuItem.lazyInit;
			menuItem.lazyInit = null;
			var result = eval(lazyInit);
			if (result)
			{
				var html = this.drawMenuItems(menuItem.firstChildIndex, menuItem.childIndentation);
				var div = document.getElementById('children'+menuItemIndex);
				div.innerHTML = html;
			}
		}
		menuItem.isOpen = !menuItem.isOpen;
		if (menuItem.isOpen && menuItem.firstChildIndex != -1)
		{
			Main.show('children'+menuItem.index);
		}
		else
		{
			Main.hide('children'+menuItem.index);
		}
		this.updateIconsAndText(menuItemIndex);
	}
	
	/**
		Open a menu item and make sure all parents are open as well.
		@param menuItemIndex The index of the menu item to open
	*/
	this.open = function(menuItemIndex, noParents)
	{
		var menuItem = this.menuItems[menuItemIndex];
		if (!menuItem || (menuItem.isOpen && noParents)) return;
		if (!menuItem.lazyInit)
		{
			menuItem.isOpen = true;
			Main.show('children'+menuItem.index);
			this.updateIconsAndText(menuItemIndex);
		}
		if (!noParents && menuItem.parentItemIndex != -1) this.open(menuItem.parentItemIndex, false);
	}
	
	/**
		Open all menu items in the joust menu.
	*/
	this.openAll = function()
	{
		for (var i = 0; i < this.menuItems.length; i++)
		{
			this.open(i, true);
		}
	}

	/**
		Close a menu item.
		@param menuItemIndex The index of the menu item to close
	*/
	this.close = function(menuItemIndex)
	{
		var menuItem = this.menuItems[menuItemIndex];
		if (!menuItem || !menuItem.isOpen) return;
		menuItem.isOpen = false;
		Main.hide('children'+menuItem.index);
		this.updateIconsAndText(menuItemIndex);
	}
	
	/**
		Open all menu items in the joust menu.
	*/
	this.closeAll = function()
	{
		for (var i = 0; i < this.menuItems.length; i++)
		{
			this.close(i);
		}
	}
	
	/**
		Select a menu item and load it's URL into the target frame. 
		Only one menu item can be selected at a time. Any previously 
		selected menu item is automatically deselected. 
		@param menuItemIndex The index of the menu item to select 
	*/
	this.select = function(menuItemIndex, event, noclickevent)
	{
		var menuItem = this.menuItems[menuItemIndex];
		if (!menuItem) return;

		if (menuItem.parentItemIndex != -1) this.open(menuItem.parentItemIndex);
		if (menuItemIndex != this.selectedItemIndex)
		{
			if (this.selectedItemIndex != -1)
			{
				var current = this.menuItems[this.selectedItemIndex];
				current.isSelected = false;
				this.updateIconsAndText(this.selectedItemIndex);
			}
			menuItem.isSelected = true;
			if (menuItem.parentItemIndex != -1) this.open(menuItem.parentItemIndex);
			this.selectedItemIndex = menuItemIndex;
			this.updateIconsAndText(menuItemIndex);
			if (this.div)
			{
				// Make sure the selected menu entry is visible, otherwise try to scroll the main <div>
				var selected = document.getElementById('menuItem'+menuItemIndex);
				if (selected.offsetTop > this.div.scrollTop + this.div.offsetHeight || 
					selected.offsetTop < this.div.scrollTop)
				{
					this.div.scrollTop = selected.offsetTop - 40;
				}
			}
			if (!noclickevent) eval(menuItem.onclick);
		}
		else if (this.alwaysSendOnClickToSelected && !noclickevent)
		{
			eval(menuItem.onclick);
		}
	}

	/**
		Update the outline and document icons and the class name of the
		selected menu item. This method is called whenever a menuitem
		is toggled or selected to replace the icons and update the
		text.
		@param menuItemIndex The index of the menu item to update
	*/
	this.updateIconsAndText = function(menuItemIndex, recursive)
	{
		var menuItem = this.menuItems[menuItemIndex];
		if (!menuItem) return;
		if (recursive)
		{
			this.updateIconsAndText(menuItem.parentItemIndex, recursive);
		}
		var outlineImg = document.getElementById('outlineIcon'+menuItem.index);
		if (outlineImg)
		{
			var outlineIcon = menuItem.getOutlineIcon();
			if (outlineIcon) outlineImg.src = outlineIcon.src;
		}
		var docImg = document.getElementById('docIcon'+menuItem.index);
		if (docImg)
		{
			var docIcon = menuItem.getDocIcon();
			if (docIcon) docImg.src = docIcon.src;
		}
		var text = document.getElementById('menuItem'+menuItem.index);
		if (text)
		{
			if (menuItem.isSelected)
			{
				Main.addClass(text, 'selected');
			}
			else
			{
				Main.removeClass(text, 'selected');
			}
		}		
	}
	
	/**
	 	Change cursor on a menu item.
	 	@param menuItemIndex Index of the menu item over which the cursor should change.
	 	@param newCursor A string value of what to set the cursor to. 
	 */
	this.changeCursorOnMenuItem = function(menuItemIndex, newCursor)
	{
		var menuItem = this.menuItems[menuItemIndex];
		document.getElementById('menuItem'+menuItem.index).style.cursor=newCursor;
		document.getElementById('outlineIcon' + menuItem.index).style.cursor=newCursor;
		document.getElementById('docIcon' + menuItem.index).style.cursor=newCursor;
	}
}

/**
	Represents a menu item
	@param menu The parent menu object
	@param index The index of this menu item in the parent menus collection: menuItems
	@param iconName The name of the document icon
	@param text The text to display
	@param onclick A string containing a JavaScript statement that is evaluated when the user clicks on the menu item
	@param tooltip A tooltip text
	@param nextItemIndex The index of the next menu item on the same level, or -1 if this is the last item
	@param previousItemIndex The index of the previous menu item on the same level, or -1 if this is the first item
	@param parentItemIndex The index of the parent menu item, or -1 if this is on the root level
	@param isOpen A flag indicating if the menu item should be open or closed to begin with
	@param lazyInit A string containing a JavaScript statement that is evaluation the first time the user opens the menu item
*/
function MenuItem(menu, index, iconName, text, onclick, tooltip, nextItemIndex, previousItemIndex, parentItemIndex, externalId, isOpen, lazyInit)
{
	this.menu = menu;
	this.index = index;
	this.iconName = iconName;
	this.text = text;
	this.onclick = onclick;
	this.tooltip = tooltip;
	this.style = null;
	this.clazz = 'menuitem';
	this.nextItemIndex = nextItemIndex;
	this.previousItemIndex = previousItemIndex;
	this.parentItemIndex = parentItemIndex;
	this.externalId = externalId;
	this.isOpen = isOpen && !lazyInit;
	this.lazyInit = lazyInit;
	this.isSelected = false;
	this.firstChildIndex = -1;
	this.lastChildIndex = -1;
	if (parentItemIndex != -1) menu.menuItems[parentItemIndex].lazyInit = null;
	
	/**
		Get the Icon object that is used to display the outline, taking
		open/closed/selected status into account.
	*/
	this.getOutlineIcon = function()
	{
		if (this.noOutlineIcon == true) return null;
		var name = 'icon';
		if (this.lazyInit)
		{
			name += 'Lazy';
		}
		else if (this.firstChildIndex != -1)
		{
			name += this.isOpen ? 'Minus' : 'Plus';
		}
		else
		{
			name += 'Join';
		}
		if (this.nextItemIndex == -1)
		{
			name += this.index == this.menu.firstItemIndex ? 'Only' : 'Bottom';
		}
		else if (this.previousItemIndex == -1 && this.parentItemIndex == -1)
		{
			name += 'Top';
		}
		return IconStore.getIcon(name);
	}
	
	/**
		Get the Icon object for the document icon, taking
		open/closed/selected status into account.
	*/
	this.getDocIcon = function()
	{
		var name = this.iconName;
		if (this.isOpen && IconStore.exists(name+'Expanded')) name += 'Expanded';
		if (this.isSelected && IconStore.exists(name+'Selected')) name += 'Selected';
		return IconStore.getIcon(name);
	}

	/**
		Generate the HTML to display this menu item. This method will not
		recurse into child items.
		@param indentString A string that is prepeneded to the HTML to give the impression
			of indentation
	*/
	this.draw = function(indentString)
	{
		var outlineIcon = this.getOutlineIcon();
		var outlineTag = outlineIcon ? outlineIcon.getImgTag('outlineIcon'+this.index) : '';
		
		var docIcon = this.getDocIcon();
		var docTag = docIcon ? docIcon.getImgTag('docIcon'+this.index) : '';

		var html = '<div id="menuItem'+this.index+'"';
		if (this.clazz) html += ' class="' + this.clazz + '"';
		if (this.style) html += ' style="' + this.style + '"';
		html += ' onclick="JoustMenu.select('+this.index+', event)"';
		html += ' onmouseover="Main.addClass(this, \'hover\')"';
		html += ' onmouseout="Main.removeClass(this, \'hover\')" title="' + this.tooltip + '">';
		html += '<table border="0" cellspacing="0" cellpadding="0"><tr><td width="10%">';
		html += indentString;
		if (this.firstChildIndex != -1 || this.lazyInit)
		{
			html += '<a href="javascript:JoustMenu.toggle('+this.index+')" onclick="event.cancelBubble=true">'+outlineTag+docTag+'</a>';
		}
		else
		{
			html += outlineTag+docTag;
		}
		html += '</td><td class="text" id="text'+this.index+'" width="90%">';
		html += '&nbsp;'+this.text+'&nbsp;';
		html += '</td></tr></table>';
		html += '</div>\n';
		return html;
	}

}

var IconStore = new IconStoreClass();
/**
	Keeps track of all images used in the outline.
*/
function IconStoreClass()
{
	this.icons = new Array();
	this.isInitialised = false;

	/**
		Initialise the IconStore object.
	*/
	this.init = function(imageDir, width, height)
	{
		var path = imageDir ? imageDir : getRoot()+'images/joust/';
		if (!width) width = 18;
		if (!height) height = 16;
		this.addIcon('iconPlusTop', path + 'plustop.gif', width, height);
		this.addIcon('iconPlus', path + 'plus.gif', width, height);
		this.addIcon('iconPlusBottom', path + 'plusbottom.gif', width, height);
		this.addIcon('iconPlusOnly', path + 'plusonly.gif', width, height);
		this.addIcon('iconMinusTop', path + 'minustop.gif', width, height);
		this.addIcon('iconMinus', path + 'minus.gif', width, height);
		this.addIcon('iconMinusBottom', path + 'minusbottom.gif', width, height);
		this.addIcon('iconMinusOnly', path + 'minusonly.gif', width, height);
		this.addIcon('iconLazyTop', path + 'lazytop.gif', width, height);
		this.addIcon('iconLazy', path + 'lazy.gif', width, height);
		this.addIcon('iconLazyBottom', path + 'lazybottom.gif', width, height);
		this.addIcon('iconLazyOnly', path + 'lazyonly.gif', width, height);
		this.addIcon('iconLine', path + 'line.gif', width, height);
		this.addIcon('iconBlank', path + 'blank.gif', width, height);
		this.addIcon('iconJoinTop', path + 'jointop.gif', width, height);
		this.addIcon('iconJoin', path + 'join.gif', width, height);
		this.addIcon('iconJoinBottom', path + 'joinbottom.gif', width, height);
		this.addIcon('Folder', path + 'folderclosed.gif', width, height);
		this.addIcon('FolderExpanded', path + 'folderopen.gif', width, height);
		this.addIcon('FolderSelected', path + 'folderselected.gif', width, height);
		this.addIcon('FolderExpandedSelected', path + 'folderselected.gif', width, height);
		this.addIcon('FolderDeleted', path + 'folderdeleted.gif', width, height);
		this.addIcon('FolderCompressed', path + 'foldercompressed.gif', width, height);
		this.addIcon('FolderCompressedExpanded', path + 'foldercompressed.gif', width, height);
		this.addIcon('FolderCompressedSelected', path + 'foldercompressedselected.gif', width, height);
		this.addIcon('FolderCompressedExpandedSelected', path + 'foldercompressedselected.gif', width, height);
		this.addIcon('Root', path + 'root.gif', width, height);
		this.addIcon('Home', path + 'home.gif', width, height);
		this.addIcon('UserHome', path + 'userhome.gif', width, height);
		this.isInitialised = true;
	}

	/**
		Add an icon to the IconStore
		@param name The name of the icon
		@param src The src URL of the image
		@param width The width of the image
		@param height The height of the image
	*/
	this.addIcon = function(name, src, width, height)
	{
		var insertIndex = this.icons.length;
		var icon = new Icon(name, src, width, height, insertIndex);
		this.icons[insertIndex] = icon;
		this.icons[name] = icon;
	}
	
	/**
		Get the Icon object with the given name.
	*/
	this.getIcon = function(name)
	{
		return this.icons[name];
	}
	
	/**
		Check if an Icon with the given name has been defined.
	*/
	this.exists = function(name)
	{
		return this.icons[name] != undefined;
	}
}

/**
	Represents a single icon,
	@param name The name of the icon
	@param src The src URL of the image
	@param width The width of the image
	@param height The height of the image
	@param index The index of this icon in the IconStore
*/
function Icon(name, src, width, height, index)
{
	this.name = name;
	this.src = src;
	this.width = width;
	this.height = height;
	this.index = index;
	// Preload the image
	this.img = new Image(width, height);
	this.img.src = src;
	
	/**
		Change the attributes of an image.
	*/
	this.setAttributes = function(src, width, height)
	{
		if (width) this.width = width;
		if (height) this.height = height;
		if (src) 
		{
			this.src = src;
			this.img = new Image(this.width, this.height);
			this.img.src = src;
		}
	}
	
	/**
		Generate a HTML <img> tag for this icon.
	*/
	this.getImgTag = function(id)
	{
		var html = '<img';
		if (id) html += ' id="'+id+'"';
		html+= ' src="'+this.src+'" width="'+this.width+'" height="'+this.height+'" border="0">';
		return html
	}
}


